      subroutine eebwbw(nwp,nwm,p1,p2,p3,p45,p6,p78,
     &                          ps1,ps2,ps3,ps45,ps6,ps78,mat2)
c----------------------------------------------------------------------c
c Returns the squared matrix element mat2 averaged over initial spins 
c            and summed over final spins and colours of
c
c      e+(p1) + e-(p2) --> b(p3) + W+(p45) + b~(p6) + W-(p78)
c
c
c Input: nwp,nwm = 2/4 if W+, W- decays into leptons/hadrons,
c        pi(0:3) - external particle momenta,
c        psi(0:4) - spherical components of the external particle momenta,
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c Massive vector bosons are taken in the unitary gauge and massless 
c in the Feynman gauge
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 15 June, 2001; modified on 28 March, 2002
c----------------------------------------------------------------------c

      use mathprms 
      use inprms
      use parproc
c      use polariz
      use drivec

      implicit none

      integer :: nwp,nwm

      real(8), dimension(0:3) :: p1,p2,p3,p45,p6,p78
      real(8), dimension(0:4) :: ps1,ps2,ps3,ps45,ps6,ps78

      real(8) :: ebeam=zero,mat2

      integer :: i,i1,i2,i3,i45,i6,i78,i145,i36,i782,i450,i60,i780

      complex(8), dimension(4) :: xp1,xp2,xp3,xp45,xp6,xp78,
     &                              yp1,yp2,yp3,yp45,yp6,yp78

      complex(8), dimension(4,2) :: u1,u2,u3,u5,u6,u7

      complex(8), dimension(4,2,2) :: ea36,ez36,
     &                                xa36,ya36,xz36,yz36,
     &                                ea12,xa12,ya12,ez12,xz12,yz12
      complex(8), dimension(2,2) :: h12,h36

      real(8), dimension(0:3,nwp) :: ewp
      real(8), dimension(0:3,nwm) :: ewm

      complex(8), dimension(4,nwp) :: ew45,xw45,yw45
      complex(8), dimension(4,nwm) :: ew78,xw78,yw78
      complex(8), dimension(4,nwm,nwp) :: eaww,ezww,
     &                                    xaww,yaww,xzww,yzww
      complex(8), dimension(4,2,nwp,2) :: e1ww2,x1ww2,y1ww2,
     &                                    e3ww6,x3ww6,y3ww6

      complex(8), dimension(4,nwm,2,2) :: e7836,x7836,y7836,
     &                                    e7812,x7812,y7812
      complex(8), dimension(4,nwp,2,2) :: e4536,x4536,y4536,
     &                                    e4512,x4512,y4512

      complex(8), dimension(4,nwm,2) :: u782,u786
      complex(8), dimension(4,2,nwp) :: u145,u345
      complex(8), dimension(4,2,2,2) :: u136,u312,u126
      complex(8), dimension(4,2,nwp,nwm) :: u14578
      complex(8), dimension(4,2,nwm,nwp) :: u1ww
      complex(8), dimension(nwp,nwm) :: h4578

      complex(8) :: mat,scc

      complex(8), dimension(2,nwp,2,2,nwm,2) :: amp1
      complex(8), dimension(2,nwm,nwp,2,2,2) :: amp2
      complex(8), dimension(2,2,nwp,2,nwm,2) :: amp3,amp16
      complex(8), dimension(2,nwp,nwm,2,2,2) :: amp4
      complex(8), dimension(2,nwp,nwm,2,2,2) :: amp5
      complex(8), dimension(2,2,2,nwp,nwm,2) :: amp6
      complex(8), dimension(2,2,2,nwm,nwp,2) :: amp7
      complex(8), dimension(2,nwp,2,2,nwm,2) :: amp8
      complex(8), dimension(2,nwp,2,2,nwm,2) :: amp9
      complex(8), dimension(nwm,2,2,nwp,2,2) :: amp10
      complex(8), dimension(nwp,2,2,nwm,2,2) :: amp11
      complex(8), dimension(nwp,nwm,2,2,2,2) :: amp12,amp13
      complex(8), dimension(2,2,2,nwm,nwp,2) :: amp14
      complex(8), dimension(2,2,2,nwp,nwm,2) :: amp15
      complex(8), dimension(2,nwm,nwp,2,2,2) :: amp17
      complex(8), dimension(2,nwp,2,2,nwm,2) :: amp18
      complex(8), dimension(2,nwp,nwm,2,2,2) :: amp19,amp20


      save ebeam,u1,u2,xp1,yp1,xp2,yp2,ea12,xa12,ya12,ez12,xz12,yz12

      if (ebeam /= p1(0)) then

       call spinornew(p1,ps1,m1,-1,1,2,u1)
       call spinornew(p2,ps2,m2,1,0,2,u2)

       call spinr(p1,xp1,yp1)
       call spinr(p2,xp2,yp2)

       call fvfa(u1,2,-p1,gll(1),gll(2),zero,ma2,zero,u2,2,-p2,
     &                                                 ea12,xa12,ya12)
       call fvfa(u1,2,-p1,zll(1),zll(2),zero,mz2,zero,u2,2,-p2,
     &                                                 ez12,xz12,yz12)

       ebeam=p1(0)

      end if


c External particle momenta in the spinor representation:

      call spinr(p3,xp3,yp3)
      call spinr(p45,xp45,yp45)
      call spinr(p6,xp6,yp6)
      call spinr(p78,xp78,yp78)

c Spinors of the external fermions:

      call spinornew(p3,ps3,m3,1,1,2,u3)
      call spinornew(p6,ps6,m6,-1,0,2,u6)

c Polarization vectors of the intermidiate W's:

      call recpol(p45,ps45,mw,ewp)
      call recpol(p78,ps78,mw,ewm)

      ew45=cmplx(ewp,zero,8)
      ew78=cmplx(ewm,zero,8)

      do i=1,nwp
       call spinc(ew45(:,i),xw45(:,i),yw45(:,i))
      end do
      do i=1,nwm
       call spinc(ew78(:,i),xw78(:,i),yw78(:,i))
      end do

C Building blocks:

      call uep(u3,xw45,yw45,wff(1),wff(2),
     &         xp3+xp45,yp3+yp45,dm3,ocom,u345,2,nwp)

      call peu(-xp6-xp78,-yp6-yp78,dm6,ocom,xw78,yw78,wff(1),wff(2),
     &         u6,u786,nwm,2)

      if (itt /= 1) then

      call uep(u1,xw45,yw45,wff(1),wff(2),
     &        -xp1+xp45,-yp1+yp45,dm1,ocom,u145,2,nwp)

      call uep(u145,xw78,yw78,wff(1),wff(2),
     &         xp2-xp3-xp6,yp2-yp3-yp6,cm2,ocom,u14578,2*nwp,nwm)

      call peu(xp2-xp78,yp2-yp78,dm2,ocom,xw78,yw78,wff(1),wff(2),
     &         u2,u782,nwm,2)

      call fvfa(u3,2,p3,gdd(1),gdd(2),zero,ma2,zero,u6,2,p6,
     &                                                 ea36,xa36,ya36)
      call fvfa(u3,2,p3,zdd(1),zdd(2),zero,mz2,zero,u6,2,p6,
     &                                                 ez36,xz36,yz36)

      if (ihiggs == 1) then
       call fhfb(u3,2,p3,hff(3),mh2,u6,2,p6,h36)
       call eeh(hww,p45+p78,mh2,ew45,nwp,ew78,nwm,h4578)
      end if

      call eev(gww,ma2,-p78,ew78,nwm,-p45,ew45,nwp,eaww,xaww,yaww)
      call eev(zww,mz2,-p78,ew78,nwm,-p45,ew45,nwp,ezww,xzww,yzww)

      call eve(onec,mw2,-p78,ew78,nwm,-p3-p6,gww*ea36+zww*ez36,4,
     &                                 ihiggs,hww,h36,e7836,x7836,y7836)

      call eve(onec,mw2,-p78,ew78,nwm,p1+p2,gww*ea12+zww*ez12,4,
     &                                 0,hww,h12,e7812,x7812,y7812)

      call vee(onec,mw2,-p45,ew45,nwp,-p3-p6,gww*ea36+zww*ez36,4,
     &                                 ihiggs,hww,h36,e4536,x4536,y4536)

      call vee(onec,mw2,-p45,ew45,nwp,p1+p2,gww*ea12+zww*ez12,4,
     &                                 0,hww,h12,e4512,x4512,y4512)

      call uep(u1,gll(1)*xaww+zll(1)*xzww,gll(2)*yaww+zll(2)*yzww,
     &         onec,onec,xp2-xp3-xp6,yp2-yp3-yp6,cm1,ocom,
     &         u1ww,2,nwm*nwp)

      call uep(u1,gll(1)*xa36+zll(1)*xz36,gll(2)*ya36+zll(2)*yz36,
     &         onec,onec,-xp1+xp3+xp6,-yp1+yp3+yp6,cm1,ocom,
     &         u136,2,4)

      call uep(u3,gdd(1)*xa12+zdd(1)*xz12,gdd(2)*ya12+zdd(2)*yz12,
     &         onec,onec,xp3-xp1-xp2,yp3-yp1-yp2,cm3,ocom,
     &         u312,2,4)

      call fvfa(u145,2*nwp,-p1+p45,wff(1),wff(2),zero,mw2,zero,u2,2,-p2,
     &                                                e1ww2,x1ww2,y1ww2)

      call fvfa(u345,2*nwp,p3+p45,wff(1),wff(2),zero,mw2,zero,u6,2,p6,
     &                                                e3ww6,x3ww6,y3ww6)

      call peu(-xp6+xp1+xp2,-yp6+xp1+xp2,cm6,ocom,
     &         gdd(1)*xa12+zdd(1)*xz12,gdd(2)*ya12+zdd(2)*yz12,
     &         onec,onec,u6,u126,4,2)

      end if ! itt

c Amplitudes:

      call fef(u345,guu(1)*xa12+zuu(1)*xz12,guu(2)*ya12+zuu(2)*yz12,
     &         onec,onec,u786,amp18,2*nwp,4,nwm*2)

      if (itt /= 1) then

      call fef(u145,xz36,yz36,znn(1),znn(2),u782,amp1,2*nwp,4,2*nwm)

      call fef(u1ww,gll(1)*xa36+zll(1)*xz36,gll(2)*ya36+zll(2)*yz36,
     &         onec,onec,u2,amp2,2*nwm*nwp,4,2)

      call fef(u3,x1ww2,y1ww2,wff(1),wff(2),u786,amp3,2,2*nwp*2,nwm*2)

      call fef(u14578,gll(1)*xa36+zll(1)*xz36,gll(2)*ya36+zll(2)*yz36,
     &         onec,onec,u2,amp4,2*nwp*nwm,4,2)

      call fef(u145,x7836,y7836,wff(1),wff(2),u2,amp5,2*nwp,nwm*4,2)

      call fef(u136,xw45,yw45,wff(1),wff(2),u782,amp6,2*4,nwp,nwm*2)

      call fef(u136,gll(1)*xaww+zll(1)*xzww,gll(2)*yaww+zll(2)*yzww,
     &         onec,onec,u2,amp7,2*4,nwm*nwp,2)

      call fef(u1,x4536,y4536,wff(1),wff(2),u782,amp8,2,nwp*4,nwm*2)

      call fef(u3,x4512,y4512,wff(1),wff(2),u786,amp9,2,nwp*4,nwm*2)

      call eee(-p1-p2+p78,e7812,-p45,ew45,-p3-p6,gww*ea36+zww*ez36,
     &         amp10,nwm*4,nwp,4)

      call eee(-p1-p2+p45,e4512,-p78,ew78,-p3-p6,gww*ea36+zww*ez36,
     &         amp11,nwp*4,nwm,4)

      call eeee(ew45,ew78,ggww*ea12+gzww*ez12,ea36,amp12,nwp,nwm,4,4)
      call eeee(ew45,ew78,gzww*ea12+zzww*ez12,ez36,amp13,nwp,nwm,4,4)

      call fef(u312,gdd(1)*xaww+zdd(1)*xzww,gdd(2)*yaww+zdd(2)*yzww,
     &         onec,onec,u6,amp14,2*4,nwm*nwp,2)

      call fef(u312,xw45,yw45,wff(1),wff(2),u786,amp15,8,nwp,nwm*2)

      call fef(u1,x3ww6,y3ww6,wff(1),wff(2),u782,amp16,2,2*nwp*2,nwm*2)

      call fef(u3,gdd(1)*xaww+zdd(1)*xzww,gdd(2)*yaww+zdd(2)*yzww,
     &         onec,onec,u126,amp17,2,nwm*nwp,8)

      call fef(u345,xw78,yw78,wff(1),wff(2),u126,amp19,2*nwp,nwm,8)

      call fef(u345,x7812,y7812,wff(1),wff(2),u6,amp20,2*nwp,nwm*4,2)

      end if ! itt

c The spin averaged squared matrix element:

      mat2=zero

      do i1=1,2
      do i2=1,2
      do i3=1,2
      do i45=1,nwp
      do i6=1,2
      do i78=1,nwm

      mat=amp18(i3,i45,i1,i2,i78,i6)                                   ! 56-57

      if (itt /= 1) then

      mat=mat
     &   +amp1(i1,i45,i3,i6,i78,i2)                                   ! 1
     &   -amp2(i1,i78,i45,i3,i6,i2)                                   ! 2-5
     &   +amp3(i3,i1,i45,i2,i78,i6)                                   ! 6
     &   +amp4(i1,i45,i78,i3,i6,i2)                                   ! 7-8
     &   -amp5(i1,i45,i78,i3,i6,i2)                                   ! 9-11
     &   +amp6(i1,i3,i6,i45,i78,i2)                                   ! 12-13
     &   -amp7(i1,i3,i6,i78,i45,i2)                                   ! 14-17
     &   -amp8(i1,i45,i3,i6,i78,i2)                                   ! 18-20
     &   -amp9(i3,i45,i1,i2,i78,i6)                                   ! 23-24
     &  -amp10(i78,i1,i2,i45,i3,i6)                            ! 25-26, 28-29
     &  +amp11(i45,i1,i2,i78,i3,i6)                            ! 31-32, 34-35
     &  -amp12(i45,i78,i1,i2,i3,i6)                                   ! 37,40
     &  -amp13(i45,i78,i1,i2,i3,i6)                                   ! 38-39
     &  -amp14(i3,i1,i2,i78,i45,i6)                            ! 41-42, 44-45
     &  +amp15(i3,i1,i2,i45,i78,i6)                                   ! 47-48
     &  +amp16(i1,i3,i45,i6,i78,i2)                                   ! 49
     &  -amp17(i3,i78,i45,i1,i2,i6)                            ! 50-51, 53-54
     &  +amp19(i3,i45,i78,i1,i2,i6)                                   ! 58-59
     &  -amp20(i3,i45,i78,i1,i2,i6)                                   ! 60-61

      if (ihiggs == 1) then
       mat=mat
     &    +hzz*h4578(i45,i78)*scc(ez12(:,i1,i2),ez36(:,i3,i6))        ! 21
     &    -hzz*h36(i3,i6)*scc(ez12(:,i1,i2),ezww(:,i78,i45))          ! 22
     &    -hww*h36(i3,i6)*scc(e7812(:,i78,i1,i2),ew45(:,i45))         ! 27,30
     &    -hww*h36(i3,i6)*scc(e4512(:,i45,i1,i2),ew78(:,i78))         ! 33,36
     &    +hff(3)*h4578(i45,i78)                                      ! 43,46
     &          *dot_product(conjg(u312(:,i3,i1,i2)),u6(:,i6))
     &    +hff(3)*h4578(i45,i78)                                      ! 52,55
     &          *dot_product(conjg(u3(:,i3)),u126(:,i1,i2,i6))
      end if
      end if ! itt

      mat2=mat2+abs(mat)**2

      end do
      end do
      end do
      end do
      end do
      end do

      mat2=colfac2*mat2

      if (ischeme == 1 .and. ialpw == 1) then
       mat2=calpw*mat2
      end if

      end 



