      subroutine eeee(ew1,ew2,ev1,ev2,mat,nw1,nw2,nv1,nv2)
c----------------------------------------------------------------------c
c Returns a contraction of the quartic gauge boson coupling with 4 
c complex four vectors:
c  mat(iw1,iw2,iv1,iv2)=ew1(iw1).ev1(iv1)*ew2(iw2).ev2(iv2)
c                      +ew1(iw1).ev2(iv2)*ew2(iw2).ev1(iv1)
c                    -2*ew1(iw1).ew2(iw2)*ev1(iv1).ev2(iv2)
c
c Input: ew1,ew2 - four vectors contracted with a W boson,
c        ev1,ev2 - four vectors contracted with a photon, Z or W boson,
c        nw1,nw2,nv1,nv2 - No's of polarization states of 
c                          ew1,ew2,ev1,ev2, respectively.
c                          
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 17 April, 1998
c----------------------------------------------------------------------c

      implicit none

      integer nw1,nw2,nv1,nv2
      complex(8), dimension(0:3,nw1) :: ew1
      complex(8), dimension(0:3,nw2) :: ew2
      complex(8), dimension(0:3,nv1) :: ev1
      complex(8), dimension(0:3,nv2) :: ev2
      complex(8), dimension(nw1,nw2,nv1,nv2) :: mat

      integer iw1,iw2,iv1,iv2
      complex(8) scc
      complex(8), dimension(nw1,nv1) :: w1v1
      complex(8), dimension(nw2,nv2) :: w2v2
      complex(8), dimension(nw1,nv2) :: w1v2
      complex(8), dimension(nw2,nv1) :: w2v1
      complex(8), dimension(nw1,nw2) :: w1w2
      complex(8), dimension(nv1,nv2) :: v1v2


      do iw1=1,nw1
        do iv1=1,nv1
          w1v1(iw1,iv1)=scc(ew1(:,iw1),ev1(:,iv1))
        end do
        do iv2=1,nv2
          w1v2(iw1,iv2)=scc(ew1(:,iw1),ev2(:,iv2))
        end do
        do iw2=1,nw2
          w1w2(iw1,iw2)=scc(ew1(:,iw1),ew2(:,iw2))
        end do
      end do

      do iv2=1,nv2
        do iw2=1,nw2
          w2v2(iw2,iv2)=scc(ew2(:,iw2),ev2(:,iv2))
        end do
        do iv1=1,nv1
          v1v2(iv1,iv2)=scc(ev1(:,iv1),ev2(:,iv2))
        end do
      end do

      do iw2=1,nw2
        do iv1=1,nv1
          w2v1(iw2,iv1)=scc(ew2(:,iw2),ev1(:,iv1))
        end do
      end do

      do iw1=1,nw1
        do iw2=1,nw2
          do iv1=1,nv1
            do iv2=1,nv2
              mat(iw1,iw2,iv1,iv2)=w1v1(iw1,iv1)*w2v2(iw2,iv2)
     &                            +w1v2(iw1,iv2)*w2v1(iw2,iv1)
     &                       -2.d0*w1w2(iw1,iw2)*v1v2(iv1,iv2)
            end do
          end do
        end do
      end do

      end 






