      subroutine fhfb(bu1,n1,p1,gh,mh2,u2,n2,p2,scal)
c----------------------------------------------------------------------c
c Returns a scalar: 
c
c            scal=gh/(q^2-mh2)*bu1*u2
c
c Input: bu1 - bar spinor,
c        -p1/+p1 - momentum of the incoming/outgoing fermion,
c        gh - a Higgs boson coupling to the fermion line,
c        q - four momentum of the intermidiate scalar boson,
c        mh2 = mh^2-i*mh*gamh - scalar boson mass squared including 
c                               the width,
c        u2 - spinor,
c        -p2/+p2 - momentum of the incoming/outgoing fermion,
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 2 July, 2001
c----------------------------------------------------------------------c

      implicit none

      integer :: i1,i2,n1,n2
      real(8) :: srr
      real(8), dimension(0:3) :: p1,p2,q
      complex(8) bu1(4,n1),gh,mh2,u2(4,n2),scal(n1,n2),nom

      q=p1+p2

      nom=gh/(cmplx(srr(q,q),0.d0,8)-mh2)

      do i1=1,n1
      do i2=1,n2

       scal(i1,i2)=nom*(bu1(1,i1)*u2(1,i2)+bu1(2,i1)*u2(2,i2)
     &                 +bu1(3,i1)*u2(3,i2)+bu1(4,i1)*u2(4,i2))

      end do ! i1
      end do ! i2

      end subroutine



