      subroutine fvfa(bu1,n1,p1,gvp,gvm,q2,mv2,alpha,u2,n2,p2,
     &                                                   eps,epsp,epsm)
c----------------------------------------------------------------------c
c Returns a four vector: 
c
c  eps_m=bu1*gamma_n*(gvm*P- + gvp*P+)*u2
c       *(-g_mn + (1-alpha)*q_m*q_n/(q^2-alpha*mv2))/(q^2-mv2)
c
c Input: bu1 - bar spinor,
c        n1 - number of polarization states of u1,
c        -p1/+p1 - momentum of the incoming/outgoing fermion,
c        gvm - a complex vector boson coupling to the fermion line of 
c              the negative chirality, P-=(1-gamma_5)/2,
c        gvp - a complex vector boson coupling to the fermion line of 
c              the positive chirality, P+=(1+gamma_5)/2,
c        q - four momentum of the intermidiate gauge boson,
c        mv2 = mv^2-i*mv*gamv - gauge boson mass squared including 
c                               the width,
c        alpha - a real gauge parameter,
c        q2 - the squared momentum transfer of the gauge boson propagator,
c        u2 - spinor,
c        -p2/+p2 - momentum of the incoming/outgoing fermion,
c        n2 - number of polarization states of u2.
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 10 November, 2000; modified 27 June, 2001
c----------------------------------------------------------------------c

      implicit none

      integer n1,n2,i1,i2
      real(8) q2,alpha,srr
      real(8), dimension(0:3) :: p1,p2,q
      complex(8) bu1(4,n1),gvm,gvp,mv2,u2(4,n2),eps(0:3,n1,n2),
     &           epsp(2,2,n1,n2),epsm(2,2,n1,n2),den,nomm,nomp,nom,src
      complex(8), parameter :: ocom=(0.d0,0.d0),icom=(0.d0,1.d0)

      q=p1+p2

      if (q2 == 0.d0) then
       den=cmplx(srr(q,q),0.d0,8)-mv2
      else
       den=cmplx(q2,0.d0,8)-mv2
      end if

      nomm=gvm/den
      nomp=gvp/den

      do i1=1,n1
      do i2=1,n2

      eps(:,i1,i2)=-nomm*(/bu1(3,i1)*u2(1,i2)+bu1(4,i1)*u2(2,i2),
     &                   -(bu1(3,i1)*u2(2,i2)+bu1(4,i1)*u2(1,i2)),
     &              +icom*(bu1(3,i1)*u2(2,i2)-bu1(4,i1)*u2(1,i2)),
     &                    -bu1(3,i1)*u2(1,i2)+bu1(4,i1)*u2(2,i2)/)

      if (gvp.ne.0.d0) then

      eps(:,i1,i2)=eps(:,i1,i2)
     &       -nomp*(/bu1(1,i1)*u2(3,i2)+bu1(2,i1)*u2(4,i2),
     &               bu1(1,i1)*u2(4,i2)+bu1(2,i1)*u2(3,i2),
     &        -icom*(bu1(1,i1)*u2(4,i2)-bu1(2,i1)*u2(3,i2)),
     &               bu1(1,i1)*u2(3,i2)-bu1(2,i1)*u2(4,i2)/)

      end if  ! gvp

      if (mv2.ne.ocom) then
       if (alpha == 0.d0) then
        nom=1.d0/mv2
       else
        nom=(1.d0-alpha)/(den+(1.d0-alpha)*mv2)
       end if
       eps(:,i1,i2)=eps(:,i1,i2)-nom*src(q,eps(:,i1,i2))*q
      end if

      call spinc(eps(:,i1,i2),epsp(:,:,i1,i2),epsm(:,:,i1,i2))

      end do ! i1
      end do ! i2

      end subroutine



