      subroutine kineett6f(it1,it2,iv1,iv2,mt1,gamt1,mt2,gamt2,
     &                     mv1,gamv1,mv2,gamv2,ecm,x,
     &                     p1,ps1,m1,p2,ps2,m2,p3,ps3,m3,p4,ps4,m4,
     &                     p5,ps5,m5,p6,ps6,m6,p7,ps7,m7,p8,ps8,m8,
     &                                                       dlips,flux)
c----------------------------------------------------------------------c
c Returns cartezian and spherical components of four momenta,
c phase space normalization, dlips, and the flux factor for a 2 --> 6
c process in the c.m.s. 
c
c                   p1 + p2 --> (p3 + p4 + p5) + (p6 + p7 + p8)
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c        psi(0) - the length of three momentum pi(1:3),
c        psi(1) - cosinus of the polar angle,
c        psi(2) - sinus of the polar angle,
c        psi(3) - cosinus of the azimuthal angle,
c        psi(4) - sinus of the azimuthal angle.
c
c Input: it1,it2 = 0/1 if the intermediate t-quarks are off/in resonance,
c        iv1 = 0/1 or 2 if the intermediate boson V1 is off/in resonance
c                                                            or massless,
c        iv2 = 0/1 or 2 if the intermediate boson V2 is off/in resonance
c                                                            or massless
c        ecm - the c.m.s. energy,
c        mt1,mt2,gamt1,gamt2 - a mass and width of the intermediate 
c                              resonance states,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        x(i) - random variables, 0 < x(i) < 1,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 3 July, 2001; modiffied on 20 July, 2001
c----------------------------------------------------------------------c

      use mathprms 

      implicit none

      integer :: it1,it2,iv1,iv2

      real(8) :: mt1,gamt1,mt2,gamt2,mv1,gamv1,mv2,gamv2,ecm,x(20),
     &           m1,m2,m3,m4,m5,m6,m7,m8,dlips,flux

      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p8
      real(8), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7,ps8

      real(8), dimension(1:3) :: v

      real(8) :: s345l,s345u,s345,ns345,ats345,ds345,sqs345,
     &           s45l,s45u,s45,ds45,sqs45,
     &           s678l,s678u,s678,ats678,ns678,ds678,sqs678,s78l,
     &           s78u,s78,sqs78,ds78,
     &           rs45,ns45,ats45,rs78,ns78,ats78,pp,pp3,pp6,m32,m62,
     &           ct,st,phi,dct,dphi,lamsq,cps,dlips45,dlips78,
     &           g,e,pp4,pp7,mv12,mv22,gvmv1,gvmv2,mr2,gtmt

      real(8) :: ecm0=0.d0

      save dct,dphi,cps,ecm0

      if (ecm /= ecm0) then

      pp=sqrt(lamsq(ecm,m1,m2))/(2.d0*ecm)

c Constant parts of the phase space and Jakobian:

      dct=2.d0
      dphi=2.d0*pi
      cps=1.d0/(2.d0*pi)**14

c Flux factor:

      flux=4.d0*ecm*pp

c Initial momenta:

      p1=(/sqrt(m1**2+pp**2),0.d0,0.d0,pp/)
      p2=(/sqrt(m2**2+pp**2),0.d0,0.d0,-pp/)

      ps1=(/pp,1.d0,0.d0,1.d0,0.d0/)
      ps2=(/pp,-1.d0,0.d0,-1.d0,0.d0/)

      ecm0=ecm

      end if

c s345=(p3+p4+p5)^2:

      s345l=(m3+m4+m5)**2
      s345u=(ecm-m6-m7-m8)**2

      if (it1 == 0) then
       ds345=s345u-s345l
       s345=ds345*x(1)+s345l
      else if (it1 == 1) then
       gtmt=gamt1*mt1
       mr2=mt1**2
       ats345=atan((s345l-mr2)/gtmt)
       ns345=gtmt/(atan((s345u-mr2)/gtmt)-ats345)
       s345=gtmt*tan(gtmt/ns345*x(1)+ats345)+mr2
       ds345=((s345-mr2)**2+gtmt**2)/ns345
       if (ds345 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else
       write(*,*)'it1 =',it1,' IS NOT ALLOWED'
       stop
      end if

      sqs345=sqrt(s345)

c s678=(p6+p7+p8)^2:

      s678l=(m6+m7+m8)**2
      s678u=(ecm-sqs345)**2

      if (s678u <= s678l) then
       dlips=0.d0
       return
      end if

      if (it2 == 0) then
       ds678=s678u-s678l
       s678=ds678*x(2)+s678l
      else if (it2 == 1) then
       gtmt=gamt2*mt2
       mr2=mt2**2
       ats678=atan((s678l-mr2)/gtmt)
       ns678=gtmt/(atan((s678u-mr2)/gtmt)-ats678)
       s678=gtmt*tan(gtmt/ns678*x(2)+ats678)+mr2
       ds678=((s678-mr2)**2+gtmt**2)/ns678
       if (ds678 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else
       write(*,*)'it2 =',it2,' IS NOT ALLOWED'
       stop
      end if

      if (s678 < s678l) then
       dlips=0.d0
       return
      else if (s678 > s678u) then
       dlips=0.d0
       return
      end if

      sqs678=sqrt(s678)

c s45=(p4+p5)^2:

      s45l=(m4+m5)**2
      s45u=(sqs345-m3)**2

      if (s45u <= s45l) then
       dlips=0.d0
       return
      end if

      if (iv1 == 0) then
       ds45=s45u-s45l
       s45=ds45*x(3)+s45l
      else if (iv1 == 1) then
       gvmv1=gamv1*mv1
       mv12=mv1**2
       ats45=atan((s45l-mv12)/gvmv1)
       ns45=gvmv1/(atan((s45u-mv12)/gvmv1)-ats45)
       s45=gvmv1*tan(gvmv1/ns45*x(3)+ats45)+mv12
       ds45=((s45-mv12)**2+gvmv1**2)/ns45
       if (ds45 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv1 == 2) then
       rs45=s45u/s45l
       ns45=1.d0/log(rs45)
       s45=s45l*rs45**x(3)
       ds45=s45/ns45
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      if (s45 < s45l) then
       dlips=0.d0
       return
      else if (s45 > s45u) then
       dlips=0.d0
       return
      end if

      sqs45=sqrt(s45)

c s78=(p7+p8)^2:

      s78l=(m7+m8)**2
      s78u=(sqs678-m6)**2

      if (s78u <= s78l) then
       dlips=0.d0
       return
      end if

      if (iv2 == 0) then
       ds78=s78u-s78l
       s78=ds78*x(4)+s78l
      else if (iv2 == 1) then
       gvmv2=gamv2*mv2
       mv22=mv2**2
       ats78=atan((s78l-mv22)/gvmv2)
       ns78=gvmv2/(atan((s78u-mv22)/gvmv2)-ats78)
       s78=gvmv2*tan(gvmv2/ns78*x(4)+ats78)+mv22
       ds78=((s78-mv22)**2+gvmv2**2)/ns78
       if (ds78 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv2 == 2) then
       rs78=s78u/s78l
       ns78=1.d0/log(rs78)
       s78=s78l*rs78**x(4)
       ds78=s78/ns78
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      if (s78 < s78l) then
       dlips=0.d0
       return
      else if (s78 > s78u) then
       dlips=0.d0
       return
      end if

      sqs78=sqrt(s78)

c p3 in the c.m.s. of 3, 4 and 5:

      ct=dct*x(5)-1.d0
      st=sqrt(1.d0-ct**2)
      phi=dphi*x(6)

      m32=m3**2
      p3(0)=(s345-s45+m32)/(2.d0*sqs345)

      if (p3(0) < m3) then
       dlips=0.d0
       return
      end if

      pp3=sqrt(p3(0)**2-m32)
      p3(1:)=pp3*(/st*cos(phi),st*sin(phi),ct/)

c p4 and p5 in their c.m.s.:

      call kinff(sqs45,m4,m5,x(7:8),p4,p5,pp4,dlips45)

      if (dlips45 <= 0.d0) then
       dlips=0.d0
       return
      end if

c p6 in the c.m.s. of 6, 7 and 8:

      ct=dct*x(9)-1.d0
      st=sqrt(1.d0-ct**2)
      phi=dphi*x(10)

      m62=m6**2
      p6(0)=(s678-s78+m62)/(2.d0*sqs678)

      if (p6(0) < m6) then
       dlips=0.d0
       return
      end if

      pp6=sqrt(p6(0)**2-m62)
      p6(1:)=pp6*(/st*cos(phi),st*sin(phi),ct/)

c p7 and p8 in their c.m.s.:

      call kinff(sqs78,m7,m8,x(11:12),p7,p8,pp7,dlips78)

      if (dlips78 <= 0.d0) then
       dlips=0.d0
       return
      end if

c p345 in the c.m.s.:

      ct=dct*x(13)-1.d0
      st=sqrt(1.d0-ct**2)
      pp=sqrt(lamsq(ecm,sqs345,sqs678))/(2.d0*ecm)

c The phase space normalization:

      dlips=cps*(pi*pp/ecm)*ds345*ds678*ds45*ds78*(pi*pp3/sqs345)
     &     *(pi*pp6/sqs678)*dlips45*dlips78

      if (dlips > zero) then

c Boost p4 and p5 back to the c.m.s. of particles 3, 4 and 5:

      e=sqrt(s45+pp3**2)
      v=-p3(1:)/e
      g=e/sqs45

      call boost(g,v,p4)
      call boost(g,v,p5)

c Boost p7 and p8 back to the c.m.s. of particles 6, 7 and 8:

      e=sqrt(s78+pp6**2)
      v=-p6(1:)/e
      g=e/sqs78

      call boost(g,v,p7)
      call boost(g,v,p8)

c Boost p3, p4 and p5 back to the c.m.s.:

      e=sqrt(s345+pp**2)
      v=pp*(/st,0.d0,ct/)/e
      g=e/sqs345

      call boost(g,v,p3)
      call boost(g,v,p4)
      call boost(g,v,p5)

c Boost p6, p7 and p8 back to the c.m.s.:

      e=sqrt(s678+pp**2)
      v=-pp*(/st,0.d0,ct/)/e
      g=e/sqs678

      call boost(g,v,p6)
      call boost(g,v,p7)
      call boost(g,v,p8)

c Spherical components of the momenta p3,...,p8:

      call spheric(p3,ps3)
      call spheric(p4,ps4)
      call spheric(p5,ps5)
      call spheric(p6,ps6)
      call spheric(p7,ps7)
      call spheric(p8,ps8)

      end if ! dlips

      end


