      subroutine peu(pp,pm,m,den,ep,em,gp,gm,u,ugen,ne,n)

c----------------------------------------------------------------------c
c Returns a generalized spinor:
c
c           ugen(ie,i)=(p+m)/den*e(ie)*(gp*P+ + gm*P-)*u(i)
c
c Input: pp,pm - intermidiate fermion momentum in the spinorial 
c                representation,
c        m - an intermidiate fermion mass including the width, relevant 
c            only if the corresponding denominator den=0,
c        den - the denominator of the fermion propagator,
c        ep,em - a generalized polarization vector in the spinorial
c                representation,
c        gp,gm - complex chirality couplings of the vector boson to 
c                the fermion, 
c        u - a spinor,
c        ne,n - numbers of polarization states.
c
c
c The gamma matrices are taken in the Weyl representation as in the 
c paper by K. Kolodziej, M. Zralek, Phys. Rev. D43 (1991) 3619
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 13 November, 2000
c----------------------------------------------------------------------c

      implicit none

      integer :: ne,n
      complex(8) :: m,den,gp,gm
      complex(8), dimension(2,2) :: pp,pm
      complex(8), dimension(2,2,ne) :: ep,em
      complex(8), dimension(4,n) :: u
      complex(8), dimension(4,ne,n) :: ugen

c Local variables:

      integer ie,i
      complex(8), parameter :: ocom=(0.d0,0.d0)
      complex(8) :: cden,cgp,cgm,mgp,mgm,emu(2),epu(2)
      real(8) p2


      if (den == ocom) then
c       p2=dble(pp(1,1)*pp(2,2)-pp(1,2)*pp(2,1))
c       cden=cmplx( p2-m**2,m*gam,8)
        cden=pp(1,1)*pp(2,2)-pp(1,2)*pp(2,1)-m**2
      else
       cden=den
      end if

      cgp=gp/cden
      cgm=gm/cden
      mgp=m*cgp
      mgm=m*cgm

      do ie=1,ne
       do i=1,n
        emu=matmul(em(:,:,ie),u(:2,i))
        ugen(:2,ie,i)=cgm*matmul(pp,emu)
        ugen(3:,ie,i)=mgm*emu
        if (gp /= ocom) then
         epu=matmul(ep(:,:,ie),u(3:,i))
         ugen(:2,ie,i)=ugen(:2,ie,i)+mgp*epu
         ugen(3:,ie,i)=ugen(3:,ie,i)+cgp*matmul(pm,epu)
        end if
       end do
      end do

      end subroutine
