      subroutine psnee1212(iv1,iv2,iv3,iv4,
     &                   mv1,gamv1,mv2,gamv2,mv3,gamv3,mv4,gamv4,ecm,
     &                     p1,m1,p2,m2,p3,m3,p4,m4,
     &                     p5,m5,p6,m6,p7,m7,p8,m8,dlips)
c----------------------------------------------------------------------c
c Returns phase space normalization, dlips, for a 2 --> 6 process
c
c                   p1 + p2 --> p3 + ((p4 + p5) + (p6 + (p7 + p8)))
c
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c
c Input: ivi = 0/1 or 2 if the intermediate state Vi is off/in resonance
c                                                            or massless,
c        ecm - the c.m.s. energy,
c        mvi - intermediate state mass,
c        gamvi - intermediate state width,
c        mi - particle masses.
c
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 19 September, 2001; modiffied on 
c----------------------------------------------------------------------c

      use mathprms 

      implicit none

      integer :: iv1,iv2,iv3,iv4

      real(8) :: mv1,gamv1,mv2,gamv2,mv3,gamv3,mv4,gamv4,ecm,
     &           m1,m2,m3,m4,m5,m6,m7,m8,dlips

      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p8
      real(8), dimension(0:4) :: ps1,ps2,ps3,ps4,ps5,ps6,ps7,ps8

      real(8) :: s,s45678l,s45678u,s45678,
     &           ds45678,sqs45678,s45l,s45u,s45,ds45,sqs45,
     &           s678l,s678u,s678,ats678,rs678,ns678,ds678,
     &           sqs678,s78l,s78u,s78,sqs78,ds78,
     &           lamsq,cps,mr2,gvmv,srr

      real(8) :: ecm0=0.d0

      save s,cps,ecm0

      if (ecm /= ecm0) then

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(2.d0*pi)**14*(pi/2.d0)**5
      s=ecm**2

      ecm0=ecm

      end if

c s45678=(p4+p5+p6+p7+p8)^2:

      s45678l=(m4+m5+m6+m7+m8)**2
      s45678u=(ecm-m3)**2
      s45678=srr(p4+p5+p6+p7+p8,p4+p5+p6+p7+p8)

      if (iv1 == 0) then
       ds45678=s45678u-s45678l
      else if (iv1 == 1) then
       gvmv=gamv1*mv1
       mr2=mv1**2
       ds45678=((s45678-mr2)**2+gvmv**2)/
     &    (gvmv/(atan((s45678u-mr2)/gvmv)-atan((s45678l-mr2)/gvmv)))
       if (ds45678 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv1 == 2) then
       ds45678=s45678*log(s45678u/s45678l)
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      sqs45678=sqrt(s45678)

c s45=(p4+p5)^2:

      s45l=(m4+m5)**2
      s45u=(sqs45678-m4-m5)**2

      if (s45u <= s45l) then
       dlips=0.d0
       return
      end if

      s45=srr(p4+p5,p4+p5)

      if (iv2 == 0) then
       ds45=s45u-s45l
      else if (iv2 == 1) then
       gvmv=gamv2*mv2
       mr2=mv2**2
       ds45=((s45-mr2)**2+gvmv**2)/
     &    (gvmv/(atan((s45u-mr2)/gvmv)-atan((s45l-mr2)/gvmv)))
       if (ds45 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv2 == 2) then
       ds45=s45*log(s45u/s45l)
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      if (s45 < s45l) then
       dlips=0.d0
       return
      else if (s45 > s45u) then
       dlips=0.d0
       return
      end if

      sqs45=sqrt(s45)

c s678=(p6+p7+p8)^2:

      s678l=(m6+m7+m8)**2
      s678u=(sqs45678-sqs45)**2

      if (s678u <= s678l) then
       dlips=0.d0
       return
      end if

      s678=srr(p6+p7+p8,p6+p7+p8)

      if (iv3 == 0) then
       ds678=s678u-s678l
      else if (iv3 == 1) then
       gvmv=gamv3*mv3
       mr2=mv3**2
       ats678=atan((s678l-mr2)/gvmv)
       ns678=gvmv/(atan((s678u-mr2)/gvmv)-ats678)
       ds678=((s678-mr2)**2+gvmv**2)/ns678
       if (ds678 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv3 == 2) then
       rs678=s678u/s678l
       ns678=1.d0/log(rs678)
       ds678=s678/ns678
      else
       write(*,*)'iv3 =',iv3,' IS NOT ALLOWED'
       stop
      end if

      if (s678 < s678l) then
       dlips=0.d0
       return
      else if (s678 > s678u) then
       dlips=0.d0
       return
      end if

      sqs678=sqrt(s678)

c s78=(p7+p8)^2:

      s78l=(m7+m8)**2
      s78u=(sqs678-m6)**2

      if (s78u <= s78l) then
       dlips=0.d0
       return
      end if

      s78=srr(p7+p8,p7+p8)

      if (iv4 == 0) then
       ds78=s78u-s78l
      else if (iv4 == 1) then
       gvmv=gamv4*mv4
       mr2=mv4**2
       ds78=((s78-mr2)**2+gvmv**2)/
     &   (gvmv/(atan((s78u-mr2)/gvmv)-atan((s78l-mr2)/gvmv)))
       if (ds78 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv4 == 2) then
       ds78=s78*log(s78u/s78l)
      else
       write(*,*)'iv4 =',iv4,' IS NOT ALLOWED'
       stop
      end if

      if (s78 < s78l) then
       dlips=0.d0
       return
      else if (s78 > s78u) then
       dlips=0.d0
       return
      end if

      sqs78=sqrt(s78)

c The phase space normalization:

      dlips=cps*sqrt(lamsq(ecm,m3,sqs45678))/s
     &     *ds45678*ds678*ds45*ds78
     &     *sqrt(lamsq(sqs678,m6,sqs78))/s678
     &     *sqrt(lamsq(sqs45678,sqs45,sqs678))/s45678
     &     *sqrt(lamsq(sqs45,m4,m5))/s45
     &     *sqrt(lamsq(sqs78,m7,m8))/s78

      end


