      subroutine psnee6f(iv1,iv2,iv3,iv4,
     &                   mv1,gamv1,mv2,gamv2,mv3,gamv3,mv4,gamv4,ecm,
     &                   p1,m1,p2,m2,p3,m3,p4,m4,
     &                   p5,m5,p6,m6,p7,m7,p8,m8,dlips)
c----------------------------------------------------------------------c
c Returns a phase space normalization, dlips, for a 2 --> 6
c process in the c.m.s. 
c
c                   p1 + p2 --> p3 + p4 + (p5 + p6 + (p7 + p8))
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c
c Input: ivi = 0/1 or 2 if the intermediate state Vi is off/in resonance
c                                                            or massless,
c        ecm - the c.m.s. energy,
c        mvi - intermediate state mass,
c        gamvi - intermediate state width,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 20 July, 2001; modiffied on 
c----------------------------------------------------------------------c

      use mathprms 

      implicit none

      integer :: iv1,iv2,iv3,iv4

      real(8) :: mv1,gamv1,mv2,gamv2,mv3,gamv3,mv4,gamv4,ecm,
     &           m1,m2,m3,m4,m5,m6,m7,m8,dlips

      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p8

      real(8) :: s,s34l,s34u,s34,ds34,sqs34,
     &           s56l,s56u,s56,ds56,sqs56,
     &           s5678l,s5678u,s5678,ds5678,
     &           sqs5678,s78l,s78u,s78,sqs78,ds78,
     &           lamsq,cps,mr2,gvmv,srr

      real(8) :: ecm0=0.d0

      save cps,s,ecm0

      if (ecm /= ecm0) then

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(2.d0*pi)**14*(pi/2.d0)**5

      s=ecm**2

      ecm0=ecm

      end if

c s34=(p3+p4)^2:

      s34l=(m3+m4)**2
      s34u=(ecm-m5-m6-m7-m8)**2
      s34=srr(p3+p4,p3+p4)

      if (iv1 == 0) then
       ds34=s34u-s34l
      else if (iv1 == 1) then
       gvmv=gamv1*mv1
       mr2=mv1**2
       ds34=((s34-mr2)**2+gvmv**2)/
     &    (gvmv/(atan((s34u-mr2)/gvmv)-atan((s34l-mr2)/gvmv)))
       if (ds34 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv1 == 2) then
       ds34=s34*log(s34u/s34l)
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      sqs34=sqrt(s34)

c s5678=(p5+p6+p7+p8)^2:

      s5678l=(m5+m6+m7+m8)**2
      s5678u=(ecm-sqs34)**2
      s5678=srr(p5+p6+p7+p8,p5+p6+p7+p8)

      if (s5678u <= s5678l) then
       dlips=0.d0
       return
      end if

      if (iv2 == 0) then
       ds5678=s5678u-s5678l
      else if (iv2 == 1) then
       gvmv=gamv2*mv2
       mr2=mv2**2
       ds5678=((s5678-mr2)**2+gvmv**2)/
     &     (gvmv/(atan((s5678u-mr2)/gvmv)-atan((s5678l-mr2)/gvmv)))
       if (ds5678 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv2 == 2) then
       ds5678=s5678*log(s5678u/s5678l)
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      if (s5678 < s5678l) then
       dlips=0.d0
       return
      else if (s5678 > s5678u) then
       dlips=0.d0
       return
      end if

      sqs5678=sqrt(s5678)

c s56=(p5+p6)^2:

      s56l=(m5+m6)**2
      s56u=(sqs5678-m7-m8)**2
      s56=srr(p5+p6,p5+p6)

      if (s56u <= s56l) then
       dlips=0.d0
       return
      end if

      if (iv3 == 0) then
       ds56=s56u-s56l
      else if (iv3 == 1) then
       gvmv=gamv3*mv3
       mr2=mv3**2
       ds56=((s56-mr2)**2+gvmv**2)/
     &      (gvmv/(atan((s56u-mr2)/gvmv)-atan((s56l-mr2)/gvmv)))
       if (ds56 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv3 == 2) then
       ds56=s56*log(s56u/s56l)
      else
       write(*,*)'iv3 =',iv3,' IS NOT ALLOWED'
       stop
      end if

      if (s56 < s56l) then
       dlips=0.d0
       return
      else if (s56 > s56u) then
       dlips=0.d0
       return
      end if

      sqs56=sqrt(s56)

c s78=(p7+p8)^2:

      s78l=(m7+m8)**2
      s78u=(sqs5678-sqs56)**2
      s78=srr(p7+p8,p7+p8)

      if (s78u <= s78l) then
       dlips=0.d0
       return
      end if

      if (iv4 == 0) then
       ds78=s78u-s78l
      else if (iv4 == 1) then
       gvmv=gamv4*mv4
       mr2=mv4**2
       ds78=((s78-mr2)**2+gvmv**2)/
     &      (gvmv/(atan((s78u-mr2)/gvmv)-atan((s78l-mr2)/gvmv)))
       if (ds78 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv4 == 2) then
       ds78=s78*log(s78u/s78l)
      else
       write(*,*)'iv4 =',iv4,' IS NOT ALLOWED'
       stop
      end if

      if (s78 < s78l) then
       dlips=0.d0
       return
      else if (s78 > s78u) then
       dlips=0.d0
       return
      end if

      sqs78=sqrt(s78)

c The phase space normalization:

      dlips=cps*sqrt(lamsq(ecm,sqs34,sqs5678))/s
     &     *ds34*ds5678*ds56*ds78
     &     *sqrt(lamsq(sqs5678,sqs56,sqs78))/s5678
     &     *sqrt(lamsq(sqs34,m3,m4))/s34
     &     *sqrt(lamsq(sqs56,m5,m6))/s56
     &     *sqrt(lamsq(sqs78,m7,m8))/s78


      end


