      subroutine psneetbff(iv1,iv2,ecm,mv1,gamv1,mv2,gamv2,
     &                     p1,m1,p2,m2,p3,m3,p4,m4,p5,m5,p6,m6,dlips)
c----------------------------------------------------------------------c
c Returns the phase space normalization, dlips for a 2 --> 4  process 
c the c.m.s. 
c
c                   p1 + p2 --> p3 + (p4 + p5 + p6)
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c
c Input: iv1 = 0/1 or 2 if the intermediate boson V1 is off/in resonance
c                                                            or massless,
c        iv2 = 0/1 or 2 if the intermediate boson V2 is off/in resonance
c                                                            or massless
c        ecm - the c.m.s. energy,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 28 May, 2001; modiffied on 
c----------------------------------------------------------------------c

      use mathprms 

      implicit none

      integer :: iv1,iv2

      real(8) :: ecm,mv1,gamv1,mv2,gamv2,m1,m2,m3,m4,m5,m6,
     &           s456,s56,dlips

      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6

      real(8) :: lamsq,srr,cps,s56max,s56min,ds56,sqs56,
     &           s456max,s456min,ds456,sqs456,mv12,mv22,gvmv1,gvmv2

      real(8) :: ecm0=0.d0

      save cps,ecm0

      if (ecm /= ecm0) then

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(2.d0*pi)**7/(4.d0*ecm)*(pi/2.d0)**2/(2.d0*ecm)*2.d0

      ecm0=ecm

      end if

      if (iv1 == 1) then
       gvmv1=gamv1*mv1
       mv12=mv1**2
      end if

      if (iv2 == 1) then
       gvmv2=gamv2*mv2
       mv22=mv2**2
      end if

      s456min=(m4+m5+m6)**2
      s456max=(ecm-m3)**2

      s456=srr(p4+p5+p6,p4+p5+p6)

      if (iv1 == 0) then
       ds456=s456max-s456min
      else if (iv1 == 1) then
       if (s456 < s456min) then
        s456=s456min
       end if
       ds456=((s456-mv12)**2+gvmv1**2)/
     &       (gvmv1/(atan((s456max-mv12)/gvmv1)
     &                         -atan((s456min-mv12)/gvmv1)))
      else if (iv1 == 2) then
       ds456=s456*log(s456max/s456min)
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      sqs456=sqrt(s456)

c p3 in the c.m.s.:

      s56min=(m5+m6)**2
      s56max=(sqs456-m4)**2

      s56=srr(p5+p6,p5+p6)

      if (iv2 == 0) then
       ds56=s56max-s56min
      else if (iv2 == 1) then
       if (s56 < s56min) then
        s56=s56min
       end if
       ds56=((s56-mv22)**2+gvmv2**2)/
     &      (gvmv2/(atan((s56max-mv22)/gvmv2)
     &                            -atan((s56min-mv22)/gvmv2)))
      else if (iv2 == 2) then
       ds56=s56*log(s56max/s56min)
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      sqs56=sqrt(s56)

c Phase space normalization and Jakobian:

      dlips=cps*sqrt(lamsq(ecm,m3,sqs456))*ds456*ds56
     &     *sqrt(lamsq(sqs456,sqs56,m4))/s456
     &     *sqrt(lamsq(sqs56,m5,m6))/s56

      end


