      subroutine psneett6f(it1,it2,iv1,iv2,mt1,gamt1,mt2,gamt2,
     &                     mv1,gamv1,mv2,gamv2,ecm,
     &                     p1,m1,p2,m2,p3,m3,p4,m4,
     &                     p5,m5,p6,m6,p7,m7,p8,m8,dlips)
c----------------------------------------------------------------------c
c Returns a phase space normalization, dlips, for a 2 --> 6
c process in the c.m.s. 
c
c                   p1 + p2 --> (p3 + p4 + p5) + (p6 + p7 + p8)
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c The three momentum p3 + p4 is generated in the xOz plane in this frame.
c
c        pi(0:3) - the usual four momentum components of the i-th
c                   particle, 
c
c Input: it1,it2 = 0/1 if the intermediate t-quarks are off/in resonance,
c        iv1 = 0/1 or 2 if the intermediate boson V1 is off/in resonance
c                                                            or massless,
c        iv2 = 0/1 or 2 if the intermediate boson V2 is off/in resonance
c                                                            or massless
c        ecm - the c.m.s. energy,
c        mt1,mt2,gamt1,gamt2 - a mass and width of the intermediate 
c                              resonance states,
c        mv1,mv2 - intermediate boson masses,
c        gamv1,gamv2 - intermediate boson widhts,
c        mi - particle masses.
c
c The frame is defined as that p1=(E1,0,0,p), p2=(E2,0,0,-p).
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 3 July, 2001; modiffied on 20 July, 2001
c----------------------------------------------------------------------c

      use mathprms 

      implicit none

      integer :: it1,it2,iv1,iv2

      real(8) :: mt1,gamt1,mt2,gamt2,mv1,gamv1,mv2,gamv2,ecm,
     &           m1,m2,m3,m4,m5,m6,m7,m8,dlips

      real(8), dimension(0:3) :: p1,p2,p3,p4,p5,p6,p7,p8

      real(8) :: s,s345l,s345u,s345,ds345,sqs345,
     &           s45l,s45u,s45,ds45,sqs45,
     &           s678l,s678u,s678,ds678,sqs678,s78l,
     &           s78u,s78,sqs78,ds78,
     &           lamsq,srr,cps,mv12,mv22,gvmv1,gvmv2,mr2,gtmt

      real(8) :: ecm0=0.d0

      save cps,ecm0,s

      if (ecm /= ecm0) then

c Constant parts of the phase space and Jakobian:

      cps=1.d0/(2.d0*pi)**14

      s=ecm**2
      ecm0=ecm

      end if

c s345=(p3+p4+p5)^2:

      s345l=(m3+m4+m5)**2
      s345u=(ecm-m6-m7-m8)**2
      s345=srr(p3+p4+p5,p3+p4+p5)

      if (it1 == 0) then
       ds345=s345u-s345l
      else if (it1 == 1) then
       gtmt=gamt1*mt1
       mr2=mt1**2
       ds345=((s345-mr2)**2+gtmt**2)/
     &      (gtmt/(atan((s345u-mr2)/gtmt)-atan((s345l-mr2)/gtmt)))
       if (ds345 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else
       write(*,*)'it1 =',it1,' IS NOT ALLOWED'
       stop
      end if

      sqs345=sqrt(s345)

c s678=(p6+p7+p8)^2:

      s678l=(m6+m7+m8)**2
      s678u=(ecm-sqs345)**2
      s678=srr(p6+p7+p8,p6+p7+p8)

      if (s678u <= s678l) then
       dlips=0.d0
       return
      end if

      if (it2 == 0) then
       ds678=s678u-s678l
      else if (it2 == 1) then
       gtmt=gamt2*mt2
       mr2=mt2**2
       ds678=((s678-mr2)**2+gtmt**2)/
     &       (gtmt/(atan((s678u-mr2)/gtmt)-atan((s678l-mr2)/gtmt)))
       if (ds678 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else
       write(*,*)'it2 =',it2,' IS NOT ALLOWED'
       stop
      end if

      if (s678 < s678l) then
       dlips=0.d0
       return
      else if (s678 > s678u) then
       dlips=0.d0
       return
      end if

      sqs678=sqrt(s678)

c s45=(p4+p5)^2:

      s45l=(m4+m5)**2
      s45u=(sqs345-m3)**2
      s45=srr(p4+p5,p4+p5)

      if (s45u <= s45l) then
       dlips=0.d0
       return
      end if

      if (iv1 == 0) then
       ds45=s45u-s45l
      else if (iv1 == 1) then
       gvmv1=gamv1*mv1
       mv12=mv1**2
       ds45=((s45-mv12)**2+gvmv1**2)/
     &      (gvmv1/(atan((s45u-mv12)/gvmv1)-atan((s45l-mv12)/gvmv1)))
       if (ds45 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv1 == 2) then
       ds45=s45*log(s45u/s45l)
      else
       write(*,*)'iv1 =',iv1,' IS NOT ALLOWED'
       stop
      end if

      if (s45 < s45l) then
       dlips=0.d0
       return
      else if (s45 > s45u) then
       dlips=0.d0
       return
      end if

      sqs45=sqrt(s45)

c s78=(p7+p8)^2:

      s78l=(m7+m8)**2
      s78u=(sqs678-m6)**2
      s78=srr(p7+p8,p7+p8)

      if (s78u <= s78l) then
       dlips=0.d0
       return
      end if

      if (iv2 == 0) then
       ds78=s78u-s78l
      else if (iv2 == 1) then
       gvmv2=gamv2*mv2
       mv22=mv2**2
       ds78=((s78-mv22)**2+gvmv2**2)/
     &      (gvmv2/(atan((s78u-mv22)/gvmv2)-atan((s78l-mv22)/gvmv2)))
       if (ds78 <= 0.d0) then
        dlips=0.d0
        return
       end if
      else if (iv2 == 2) then
       ds78=s78*log(s78u/s78l)
      else
       write(*,*)'iv2 =',iv2,' IS NOT ALLOWED'
       stop
      end if

      if (s78 < s78l) then
       dlips=0.d0
       return
      else if (s78 > s78u) then
       dlips=0.d0
       return
      end if

      sqs78=sqrt(s78)

c The phase space normalization:

      dlips=cps*pi*sqrt(lamsq(ecm,sqs345,sqs678))/(2.d0*s)
     &     *ds345*ds678*ds45*ds78
     &     *pi*sqrt(lamsq(sqs345,sqs45,m3))/(2.d0*s345)
     &     *pi*sqrt(lamsq(sqs678,sqs78,m6))/(2.d0*s678)
     &     *pi*sqrt(lamsq(sqs45,m4,m5))/(2.d0*s45)
     &     *pi*sqrt(lamsq(sqs78,m7,m8))/(2.d0*s78)


      end


