      subroutine spinornew(p,ps,mass,ip,ib,nhel,sp)
c----------------------------------------------------------------------c
c Returns helicity spinors in the Weyl representation.
c Input: p(0:3) - particle four mometum,
c        ps(0:4) - spherical components of the particle four mometum,
c        mass - particle mass,
c        ip = 1 for a particle (u spinor), 
c            -1 for an antiparticle (v spinor),
c        ib = 0 for a spinor,
c             1 for a bar spinor,
c        nhel=1 or 2 - the number of helicity states.
c The spinors are defined as in the paper by K. Kolodziej, M. Zralek, 
c Phys. Rev. D43 (1991) 3619:
c             sp(:,1) corresponds to sigma = -1,
c             sp(:,2) corresponds to sigma = +1.
c The Pauli spinors used in the program include extra phase factors:
c             exp(-i*phi/2) for sigma = -1,
c             exp(+i*phi/2) for sigma = +1
c with respect to the corresponding spinors defined in the paper.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 4 January, 1999
c----------------------------------------------------------------------c

      implicit none

      integer ip,ib,nhel
      real(8) p(0:3),ps(0:4),mass,cth,sth,epp,emp
      complex(8) sp(4,nhel),chi(2,nhel),eppcom,empcom
      complex(8), parameter :: ocom=(0.d0,0.d0)


      if (mass.ne.0.d0) then

      epp=sqrt(p(0)+ps(0))
      emp=mass/epp

      if (abs(ps(1)).lt.1.d0) then

         cth=sqrt((1.d0+ps(1))/2.d0)
         sth=sqrt((1.d0-ps(1))/2.d0)
         chi(:,1)=(/-sth*cmplx(ps(3),-ps(4),8),cmplx(cth,0.d0,8)/)
         chi(:,2)=(/chi(2,1),-conjg(chi(1,1))/)

         if (ib.eq.0) then

         if (ip.eq.1) then
            sp(:,1)=(/epp*chi(1,1),epp*chi(2,1),
     &                             emp*chi(1,1),emp*chi(2,1)/)
            sp(:,2)=(/emp*chi(1,2),emp*chi(2,2),
     &                             epp*chi(1,2),epp*chi(2,2)/)
         else ! ip=-1
            sp(:,1)=(/emp*chi(1,2),emp*chi(2,2),
     &                            -epp*chi(1,2),-epp*chi(2,2)/)
            sp(:,2)=(/-epp*chi(1,1),-epp*chi(2,1),
     &                             emp*chi(1,1),emp*chi(2,1)/)
         end if ! ip

         else ! ib=1

         if (ip.eq.1) then
            sp(:,1)=(/-emp*chi(2,2),emp*chi(1,2),
     &                             -epp*chi(2,2),epp*chi(1,2)/)
            sp(:,2)=(/epp*chi(2,1),-epp*chi(1,1),
     &                              emp*chi(2,1),-emp*chi(1,1)/)
         else ! ip=-1
            sp(:,1)=(/-epp*chi(2,1),epp*chi(1,1),
     &                              emp*chi(2,1),-emp*chi(1,1)/)
            sp(:,2)=(/-emp*chi(2,2),emp*chi(1,2),
     &                              epp*chi(2,2),-epp*chi(1,2)/)
         end if ! ip

         end if ! ib

      else ! abs(ps(1))=1

         eppcom=cmplx(epp,0.d0,8)
         empcom=cmplx(emp,0.d0,8)

         if (ps(1).eq.1.d0) then

            if (ib.eq.0) then

            if (ip.eq.1) then
               sp(:,1)=(/ocom,eppcom,ocom,empcom/)
               sp(:,2)=(/empcom,ocom,eppcom,ocom/)
            else ! ip=-1
               sp(:,1)=(/empcom,ocom,-eppcom,ocom/)
               sp(:,2)=(/ocom,-eppcom,ocom,empcom/)
            end if ! ip

            else ! ib=1

            if (ip.eq.1) then
               sp(:,1)=(/ocom,empcom,ocom,eppcom/)
               sp(:,2)=(/eppcom,ocom,empcom,ocom/)
            else ! ip=-1
               sp(:,1)=(/-eppcom,ocom,empcom,ocom/)
               sp(:,2)=(/ocom,empcom,ocom,-eppcom/)
            end if ! ip

            end if ! ib

         else ! ps(1)=-1

            if (ib.eq.0) then

            if (ip.eq.1) then
               sp(:,1)=(/eppcom,ocom,empcom,ocom/)
               sp(:,2)=(/ocom,-empcom,ocom,-eppcom/)
            else ! ip=-1
               sp(:,1)=(/ocom,-empcom,ocom,eppcom/)
               sp(:,2)=(/-eppcom,ocom,empcom,ocom/)
            end if ! ip

            else ! ib=1

            if (ip.eq.1) then
               sp(:,1)=(/empcom,ocom,eppcom,ocom/)
               sp(:,2)=(/ocom,-eppcom,ocom,-empcom/)
            else ! ip=-1
               sp(:,1)=(/ocom,eppcom,ocom,-empcom/)
               sp(:,2)=(/empcom,ocom,-eppcom,ocom/)
            end if ! ip

            end if ! ib

         end if ! ps(1)

      end if ! abs(ps(1))

      else ! m=0

         epp=sqrt(2.d0*p(0))

      if (abs(ps(1)).lt.1.d0) then

         cth=sqrt((1.d0+ps(1))/2.d0)
         sth=sqrt((1.d0-ps(1))/2.d0)
         if (nhel.eq.2) then
            chi(:,1)=(/-sth*cmplx(ps(3),-ps(4),8),cmplx(cth,0.d0,8)/)
            chi(:,2)=(/chi(2,1),-conjg(chi(1,1))/)
         else ! nhel=1
            chi(:,1)=(/-sth*cmplx(ps(3),-ps(4),8),cmplx(cth,0.d0,8)/)
         end if ! nhel

         if (ib.eq.0) then

         if (ip.eq.1) then
            if (nhel.eq.2) then
               sp(:,1)=(/epp*chi(1,1),epp*chi(2,1),ocom,ocom/)
               sp(:,2)=(/ocom,ocom,epp*chi(1,2),epp*chi(2,2)/)
            else ! nhel=1
               sp(:,1)=(/epp*chi(1,1),epp*chi(2,1),ocom,ocom/)
            end if ! nhel
         else ! ip=-1
            if (nhel.eq.2) then
               sp(:,1)=(/ocom,ocom,-epp*chi(1,2),-epp*chi(2,2)/)
               sp(:,2)=(/-epp*chi(1,1),-epp*chi(2,1),ocom,ocom/)
            else ! nhel=1
               sp(:,1)=(/-epp*chi(1,1),-epp*chi(2,1),ocom,ocom/)
            end if ! nhel
         end if ! ip

         else ! ib=1

         if (nhel.eq.1) then
            chi(:,1)=(/-sth*cmplx(ps(3),ps(4),8),cmplx(cth,0.d0,8)/)
         end if ! nhel

         if (ip.eq.1) then
            if (nhel.eq.2) then
               sp(:,1)=(/ocom,ocom,-epp*chi(2,2),epp*chi(1,2)/)
               sp(:,2)=(/epp*chi(2,1),-epp*chi(1,1),ocom,ocom/)
            else ! nhel=1
               sp(:,1)=(/ocom,ocom,epp*chi(1,1),epp*chi(2,1)/)
            end if ! nhel
         else ! ip=-1
            if (nhel.eq.2) then
               sp(:,1)=(/-epp*chi(2,1),epp*chi(1,1),ocom,ocom/)
               sp(:,2)=(/ocom,ocom,epp*chi(2,2),-epp*chi(1,2)/)
            else ! nhel=1
               sp(:,1)=(/ocom,ocom,-epp*chi(1,1),-epp*chi(2,1)/)
            end if ! nhel
         end if ! ip

         end if ! ib

      else ! abs(ps(1))=1

         eppcom=cmplx(epp,0.d0,8)

         if (ps(1).eq.1.d0) then

            if (ib.eq.0) then

            if (ip.eq.1) then
               if (nhel.eq.2) then
                  sp(:,1)=(/ocom,eppcom,ocom,ocom/)
                  sp(:,2)=(/ocom,ocom,eppcom,ocom/)
               else ! nhel=1
                  sp(:,1)=(/ocom,eppcom,ocom,ocom/)
               end if ! nhel
            else ! ip=-1
               if (nhel.eq.2) then
                  sp(:,1)=(/ocom,ocom,-eppcom,ocom/)
                  sp(:,2)=(/ocom,-eppcom,ocom,ocom/)
               else ! nhel=1
                  sp(:,1)=(/ocom,-eppcom,ocom,ocom/)
               end if ! nhel
            end if ! ip

            else ! ib=1

            if (ip.eq.1) then
               if (nhel.eq.2) then
                  sp(:,1)=(/ocom,ocom,ocom,eppcom/)
                  sp(:,2)=(/eppcom,ocom,ocom,ocom/)
               else ! nhel=1
                  sp(:,1)=(/ocom,ocom,ocom,eppcom/)
               end if ! nhel
            else ! ip=-1
               if (nhel.eq.2) then
                  sp(:,1)=(/-eppcom,ocom,ocom,ocom/)
                  sp(:,2)=(/ocom,ocom,ocom,-eppcom/)
               else ! nhel=1
                  sp(:,1)=(/ocom,ocom,ocom,-eppcom/)
               end if ! nhel
            end if ! ip

            end if ! ib

         else ! ps(1)=-1

            if (ib.eq.0) then

            if (ip.eq.1) then
               if (nhel.eq.2) then
                  sp(:,1)=(/eppcom,ocom,ocom,ocom/)
                  sp(:,2)=(/ocom,ocom,ocom,-eppcom/)
               else ! nhel=1
                  sp(:,1)=(/eppcom,ocom,ocom,ocom/)
               end if ! nhel
            else ! ip=-1
               if (nhel.eq.2) then
                  sp(:,1)=(/ocom,ocom,ocom,eppcom/)
                  sp(:,2)=(/-eppcom,ocom,ocom,ocom/)
               else ! nhel=1
                  sp(:,1)=(/-eppcom,ocom,ocom,ocom/)
               end if ! nhel
            end if ! ip

            else ! ib=1

            if (ip.eq.1) then
               if (nhel.eq.2) then
                  sp(:,1)=(/ocom,ocom,eppcom,ocom/)
                  sp(:,2)=(/ocom,-eppcom,ocom,ocom/)
               else ! nhel=1
                  sp(:,1)=(/ocom,ocom,eppcom,ocom/)
               end if ! nhel
            else ! ip=-1
               if (nhel.eq.2) then
                  sp(:,1)=(/ocom,eppcom,ocom,ocom/)
                  sp(:,2)=(/ocom,ocom,-eppcom,ocom/)
               else ! nhel=1
                  sp(:,1)=(/ocom,ocom,-eppcom,ocom/)
               end if ! nhel
            end if ! ip

            end if ! ib

         end if ! ps(1)

      end if ! abs(ps(1))

      end if ! m

      end subroutine

