      subroutine wwidth(m1,m2,mw,nc,ckm2,ew,sw2,width)
c----------------------------------------------------------------------c
c Returns the partial width of the W-boson averaged over initial spins 
c            and summed over final spins and colours for 
c
c                     W  --> up(m1) + down(m2)
c
c Input: m1 - mass of the up fermion,
c        m2 - mass of the down fermion,
c        mw - W mass,
c        nc = 1 for leptons,
c           = 3 for quarks,
c        ckm2 - the absolute value squared of hte corresponing CKM 
c               matrix element,
c        ew - electric charg at 2*mw,
c        sw2 - the electroweak mixing parameter.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 11 September, 1998
c----------------------------------------------------------------------c
      implicit none

      real(8) m1,m2,mw,ckm2,ew,sw2,width,pi,mw2,nc


      pi=4.d0*atan(1.d0)
      mw2=mw**2

      width=ew**2/(4.d0*pi)/12.d0*mw/sw2*nc*ckm2
     &     *sqrt((mw-m1-m2)*(mw+m1+m2)*(mw-m1+m2)*(mw+m1-m2))/mw2
     &     *(1.d0-(m1**2+m2**2)/(2.d0*mw2)-((m1**2-m2**2)/mw2)**2/2.d0)

      end



